@@ -48,7 +48,7 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
                     map.insert(
                         item.ident.clone(),
                         ts!(
-                            cm.fuzzcheck_traits_Mutator "<()>"
+                            cm.TupleMutator "< () ," Tuplei(0) " >"
                         ),
                     );
                     bindings.insert(item.ident.clone(), vec![]);
@@ -92,13 +92,9 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
     ));
 
     let pattern_match_binding_append = ident!("__proc_macro__binding__");
-    let item_pattern_match_bindings_to_tuple = |item_ident, mutable| {
+    let item_pattern_match_bindings_to_tuple = |item_ident, _mutable| {
         if item_fields[item_ident].is_empty() {
-            if mutable {
-                ts!("&mut ()")
-            } else {
-                ts!("&()")
-            }
+            ts!("()")
         } else {
             ts!("("
                 join_ts!(item_pattern_match_bindings[item_ident].iter(), binding,
@@ -123,7 +119,7 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
 
     extend_ts!(tb,
     "
-    #[derive(" cm.Debug "," cm.Clone ", " cm.PartialEq ")]
+    #[derive(" cm.Clone ")]
     #[doc(hidden)]
     pub enum " EnumSingleVariant single_variant_generics.removing_eq_type() "{"
     join_ts!(&enu.items, item,
@@ -134,23 +130,34 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
     impl " impl_mutator_generics.removing_eq_type() cm.fuzzcheck_traits_Mutator "<" enu.ident enum_generics_no_bounds "> 
         for " EnumSingleVariant single_variant_generics.removing_bounds_and_eq_type() impl_mutator_where_clause 
     "{
+        #[doc(hidden)]
         type Cache = " EnumSingleVariant
             single_variant_generics.mutating_type_params(|tp| {
                 tp.type_ident = ts!(tp.type_ident "::Cache")
             }) ";
+        #[doc(hidden)]
         type MutationStep = " EnumSingleVariant
             single_variant_generics.mutating_type_params(|tp| {
                 tp.type_ident = ts!(tp.type_ident "::MutationStep")
             }) ";
+        #[doc(hidden)]
         type ArbitraryStep = " EnumSingleVariant
             single_variant_generics.mutating_type_params(|tp| {
                 tp.type_ident = ts!(tp.type_ident "::ArbitraryStep")
             }) ";
+        #[doc(hidden)]
         type UnmutateToken = " EnumSingleVariant
             single_variant_generics.mutating_type_params(|tp| {
                 tp.type_ident = ts!(tp.type_ident "::UnmutateToken")
             }) ";
         
+        #[doc(hidden)]
+        type RecursingPartIndex = " EnumSingleVariant
+            single_variant_generics.mutating_type_params(|tp| {
+                tp.type_ident = ts!(tp.type_ident "::RecursingPartIndex")
+            }) ";
+
+        #[doc(hidden)]
         #[no_coverage]
         fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
             match self {"
@@ -160,19 +167,35 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             "}
         }
 
+        #[doc(hidden)]
         #[no_coverage]
-        fn validate_value(&self, value: &" enu.ident enum_generics_no_bounds ") -> " cm.Option "<(Self::Cache, Self::MutationStep)> {
+        fn validate_value(&self, value: &" enu.ident enum_generics_no_bounds ") -> " cm.Option "<Self::Cache> {
             match (self, value) {"
             join_ts!(&enu.items, item,
                 "(" EnumSingleVariant "::" item.ident "(m)," item.pattern_match(&enu.ident, Some(pattern_match_binding_append.clone())) ") => {
-                    m.validate_value(" item_pattern_match_bindings_to_tuple(&item.ident, false) ").map(#[no_coverage] |(x, y)| {
-                        (" EnumSingleVariant "::" item.ident "(x), " EnumSingleVariant "::" item.ident "(y))
-                    })
+                    m.validate_value(" item_pattern_match_bindings_to_tuple(&item.ident, false) ").map(" EnumSingleVariant "::" item.ident ")
                 }"
             )" _ => " cm.None ",
             }
         }
-        
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn default_mutation_step(&self, value: &" enu.ident enum_generics_no_bounds ", cache: &Self::Cache) -> Self::MutationStep {
+            match (self, value, cache) {"
+            join_ts!(&enu.items, item,
+                "(
+                    " EnumSingleVariant ":: " item.ident " (m) ,
+                    " item.pattern_match(&enu.ident, Some(pattern_match_binding_append.clone())) ",
+                    " EnumSingleVariant ":: " item.ident " (c) 
+                 ) => {
+                     " EnumSingleVariant "::" item.ident "(m.default_mutation_step(" item_pattern_match_bindings_to_tuple(&item.ident, false) ", c))
+                 }"
+            )   "_ => unreachable!()
+            }
+        }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn max_complexity(&self) -> f64 {
             match self {"
@@ -181,6 +204,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )"
             }
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn min_complexity(&self) -> f64 {
             match self {"
@@ -189,6 +214,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )"
             }
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn complexity(&self, value: &" enu.ident enum_generics_no_bounds ", cache: &Self::Cache) -> f64 {
             match (self, value, cache) {"
@@ -203,6 +230,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )   "_ => unreachable!()
             }
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(" enu.ident enum_generics_no_bounds ", f64)> {
             match (self, step) {"
@@ -220,6 +249,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             ) "_ => unreachable!()
             }
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn random_arbitrary(&self, max_cplx: f64) -> (" enu.ident enum_generics_no_bounds ", f64) {
             match self {"
@@ -233,6 +264,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
                 }"
             )"}
         }
+        
+        #[doc(hidden)]
         #[no_coverage]
         fn ordered_mutate(
             &self,
@@ -255,6 +288,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )" _ => unreachable!(),
             }
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn random_mutate(&self, value: &mut " enu.ident enum_generics_no_bounds ", cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
             match (self, value, cache) {"
@@ -272,6 +307,8 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )   "_ => unreachable!()"
             "}
         }
+
+        #[doc(hidden)]
         #[no_coverage]
         fn unmutate(&self, value: &mut " enu.ident enum_generics_no_bounds ", cache: &mut Self::Cache, t: Self::UnmutateToken) {
             match (self, value, cache, t) {"
@@ -287,6 +324,41 @@ pub fn make_single_variant_mutator(tb: &mut TokenBuilder, enu: &Enum) {
             )" _ => unreachable!()
             }
         }
+
+        #[doc(hidden)]
+        #[no_coverage]
+        fn default_recursing_part_index(&self, value: &" enu.ident enum_generics_no_bounds ", cache: &Self::Cache) -> Self::RecursingPartIndex {
+            match (self, value, cache) {"
+                join_ts!(&enu.items, item,
+                    "(" EnumSingleVariant "::" item.ident "(m), " 
+                    item.pattern_match(&enu.ident, Some(pattern_match_binding_append.clone())) ", " 
+                    EnumSingleVariant "::" item.ident "(c)" ") =>" 
+                    EnumSingleVariant "::" item.ident "(m.default_recursing_part_index(" item_pattern_match_bindings_to_tuple(&item.ident, true) ", c)),"
+                )"
+                _ => unreachable!()
+            }
+        }
+        #[doc(hidden)]
+        #[no_coverage]
+        fn recursing_part<'a, ___V, ___N>(&self, parent: &___N, value: &'a " enu.ident enum_generics_no_bounds ", index: &mut Self::RecursingPartIndex) -> " cm.Option "<&'a ___V>
+        where
+            ___V: " cm.Clone " + 'static,
+            ___N: " cm.fuzzcheck_traits_Mutator "<___V>,
+        {
+            match (self, value, index) {"
+            join_ts!(&enu.items, item,
+                "(
+                    " EnumSingleVariant "::" item.ident "(m) ,
+                    " item.pattern_match(&enu.ident, Some(pattern_match_binding_append.clone())) ",
+                    " EnumSingleVariant "::" item.ident "(index) ,
+                ) => {"
+                    "
+                    m.recursing_part::<___V, ___N>(parent, " item_pattern_match_bindings_to_tuple(&item.ident, true) ", index)
+                    "
+                "}"
+            )" _ => unreachable!()
+            }
+        }
     }
     ");
 }
