@@ -136,6 +136,7 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
     let NameMutatorMutationStep = ident!(NameMutator "MutationStep");
     let NameMutatorArbitraryStep = ident!(NameMutator "ArbitraryStep");
     let NameMutatorUnmutateToken = ident!(NameMutator "UnmutateToken");
+    let NameMutatorRecursingPartIndex = ident!(NameMutator "RecursingPartIndex");
 
     let helper_type = |helper_type: &str, conformances: bool| {
         let InnerType = ts!(
@@ -145,7 +146,7 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
         ts!(
             "#[doc(hidden)]"
             visibility "struct" ident!(NameMutator helper_type) NameMutator_generics.removing_eq_type() NameMutator_where_clause "{
-            pub inner : " if settings.recursive { ts!(cm.Box "<") } else { ts!("") } InnerType if settings.recursive { ">" } else { "" } ",
+                inner : " if settings.recursive { ts!(cm.Box "<") } else { ts!("") } InnerType if settings.recursive { ">" } else { "" } ",
             }
             impl " NameMutator_generics.removing_eq_type() ident!(NameMutator helper_type) NameMutator_generics.removing_bounds_and_eq_type() NameMutator_where_clause "{
                 #[no_coverage]
@@ -159,11 +160,7 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
             }"
             if conformances {
                 let clone_where_clause = NameMutator_where_clause.clone();
-                let mut debug_where_clause = NameMutator_where_clause.clone();
-                debug_where_clause.add_clause_items(ts!(InnerType ":" cm.Debug));
 
-                let mut partialeq_where_clause = NameMutator_where_clause.clone();
-                partialeq_where_clause.add_clause_items(ts!(InnerType ":" cm.PartialEq));
                 ts!(
                     "impl" NameMutator_generics.removing_eq_type() cm.Clone "for" ident!(NameMutator helper_type) NameMutator_generics.removing_bounds_and_eq_type() clone_where_clause "{
                         #[no_coverage]
@@ -171,18 +168,6 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
                             Self::new(self.inner " if settings.recursive { ".as_ref()" } else { "" } ".clone())
                         }
                     }
-                    impl" NameMutator_generics.removing_eq_type() cm.Debug "for" ident!(NameMutator helper_type) NameMutator_generics.removing_bounds_and_eq_type() debug_where_clause "{
-                        #[no_coverage]
-                        fn fmt(&self, f: &mut ::std::fmt::Formatter<'_>) -> ::std::result::Result<(), ::std::fmt::Error> {
-                            self.inner.fmt(f)
-                        }
-                    }
-                    impl" NameMutator_generics.removing_eq_type() cm.PartialEq "for" ident!(NameMutator helper_type) NameMutator_generics.removing_bounds_and_eq_type() partialeq_where_clause "{
-                        #[no_coverage]
-                        fn eq(&self, other: &Self) -> bool {
-                            self.inner.eq(&other.inner)
-                        }
-                    }
                     "
                 )
             } else {
@@ -192,16 +177,23 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
     };
 
     let InnerMutator_as_Mutator = ts!("<" InnerMutator "as" cm.fuzzcheck_traits_Mutator "<" type_ident type_generics.removing_bounds_and_eq_type() "> >" );
+    let documentation = proc_macro2::Literal::string(&format!(
+        "A mutator for [`{}`] 
 
+Generated by a procedural macro of [`fuzzcheck`]",
+        type_ident
+    ));
     ts!(
+    "#[doc = " documentation " ]"
     visibility "struct" NameMutator NameMutator_generics NameMutator_where_clause
     "{
-        pub mutator:" InnerMutator "
+        mutator:" InnerMutator "
     }"
     helper_type("Cache", true)
     helper_type("MutationStep", true)
     helper_type("ArbitraryStep", true)
     helper_type("UnmutateToken", false)
+    helper_type("RecursingPartIndex", true)
 
     "impl " NameMutator_generics NameMutator NameMutator_generics.removing_bounds_and_eq_type() NameMutator_where_clause "
     {"
@@ -216,36 +208,57 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
         impl " NameMutator_generics cm.fuzzcheck_traits_Mutator "<" type_ident type_generics.removing_bounds_and_eq_type() "> 
             for " NameMutator NameMutator_generics.removing_bounds_and_eq_type() NameMutator_where_clause "
         {
+            #[doc(hidden)]
             type Cache =" NameMutatorCache NameMutator_generics.removing_bounds_and_eq_type() ";
+            #[doc(hidden)]
             type MutationStep =" NameMutatorMutationStep NameMutator_generics.removing_bounds_and_eq_type() ";
+            #[doc(hidden)]
             type ArbitraryStep =" NameMutatorArbitraryStep NameMutator_generics.removing_bounds_and_eq_type() ";
+            #[doc(hidden)]
             type UnmutateToken =" NameMutatorUnmutateToken NameMutator_generics.removing_bounds_and_eq_type() ";
-        
+            #[doc(hidden)]
+            type RecursingPartIndex =" NameMutatorRecursingPartIndex NameMutator_generics.removing_bounds_and_eq_type() ";
+
+            #[doc(hidden)]
             #[no_coverage]
             fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
                 Self::ArbitraryStep::new(" InnerMutator_as_Mutator "::default_arbitrary_step(&self.mutator))
             }
 
+            #[doc(hidden)]
             #[no_coverage]
-            fn validate_value(&self, value: &" type_ident type_generics.removing_bounds_and_eq_type() ") -> " cm.Option "<(Self::Cache, Self::MutationStep)> {
-                if let " cm.Some "((c, s)) = " InnerMutator_as_Mutator "::validate_value(&self.mutator, value) {
-                    " cm.Some "((Self::Cache::new(c) , Self::MutationStep::new(s) ))
+            fn validate_value(&self, value: &" type_ident type_generics.removing_bounds_and_eq_type() ") -> " cm.Option "<Self::Cache> {
+                if let " cm.Some "(c) = " InnerMutator_as_Mutator "::validate_value(&self.mutator, value) {
+                    " cm.Some "(Self::Cache::new(c))
                 } else {
                     " cm.None "
                 }
             }
+            #[doc(hidden)]
+            #[no_coverage]
+            fn default_mutation_step(&self, value: &" type_ident type_generics.removing_bounds_and_eq_type() ", cache: &Self::Cache) -> Self::MutationStep {
+                Self::MutationStep::new(" InnerMutator_as_Mutator "::default_mutation_step(&self.mutator, value, &cache.inner))
+            }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn max_complexity(&self) -> f64 {
                 " InnerMutator_as_Mutator "::max_complexity(&self.mutator)
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn min_complexity(&self) -> f64 {
                 " InnerMutator_as_Mutator "::min_complexity(&self.mutator)
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn complexity(&self, value: &" type_ident type_generics.removing_bounds_and_eq_type() ", cache: &Self::Cache) -> f64 {
                 " InnerMutator_as_Mutator "::complexity(&self.mutator, value, &cache.inner)
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn ordered_arbitrary(&self, step: &mut Self::ArbitraryStep, max_cplx: f64) -> Option<(" type_ident type_generics.removing_bounds_and_eq_type() ", f64)> {
                 if let " cm.Some "((value, cplx)) = " InnerMutator_as_Mutator "::ordered_arbitrary(&self.mutator, &mut step.inner, max_cplx) {"
@@ -254,11 +267,15 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
                 cm.None
             "}
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn random_arbitrary(&self, max_cplx: f64) -> (" type_ident type_generics.removing_bounds_and_eq_type() ", f64) {
                 let (value, cplx) = " InnerMutator_as_Mutator "::random_arbitrary(&self.mutator, max_cplx) ;
                 (value, cplx)
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn ordered_mutate(
                 &self,
@@ -279,18 +296,36 @@ pub(crate) fn make_mutator_type_and_impl(params: CreateWrapperMutatorParams) ->
                     cm.None
                 "}
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn random_mutate(&self, value: &mut " type_ident type_generics.removing_bounds_and_eq_type() ", cache: &mut Self::Cache, max_cplx: f64) -> (Self::UnmutateToken, f64) {
                 let (t, c) =" InnerMutator_as_Mutator "::random_mutate(&self.mutator, value, &mut cache.inner, max_cplx);
                 (Self::UnmutateToken::new(t), c)
             }
+
+            #[doc(hidden)]
             #[no_coverage]
             fn unmutate(&self, value: &mut " type_ident type_generics.removing_bounds_and_eq_type() ", cache: &mut Self::Cache, t: Self::UnmutateToken) {
                 " InnerMutator_as_Mutator "::unmutate(&self.mutator, value, &mut cache.inner," if settings.recursive {
                 "*t.inner"
-            } else {
-                "t.inner"
-            }")
+                } else {
+                    "t.inner"
+                }")
+            }
+            #[doc(hidden)]
+            #[no_coverage]
+            fn default_recursing_part_index(&self, value: & " type_ident type_generics.removing_bounds_and_eq_type() ", cache: &Self::Cache) -> Self::RecursingPartIndex {
+                Self::RecursingPartIndex::new(" InnerMutator_as_Mutator "::default_recursing_part_index(&self.mutator, value, &cache.inner))
+            }
+            #[doc(hidden)]
+            #[no_coverage]
+            fn recursing_part<'a, ___V, ___N>(&self, parent: &___N, value: &'a " type_ident type_generics.removing_bounds_and_eq_type() ", index: &mut Self::RecursingPartIndex) -> " cm.Option "<&'a ___V>
+            where
+                ___V: " cm.Clone " + 'static,
+                ___N: " cm.fuzzcheck_traits_Mutator "<___V>,
+            {
+                " InnerMutator_as_Mutator "::recursing_part::<___V, ___N>(&self.mutator, parent, value, &mut index.inner)
             }
         }"
         if settings.default {
