@@ -1,5 +1,6 @@
 use decent_synquote_alternative as synquote;
-use proc_macro2::{Ident, Span, TokenStream};
+use proc_macro2::Literal;
+use proc_macro2::{Ident, Span};
 
 use synquote::parser::*;
 use synquote::token_builder::*;
@@ -32,8 +33,10 @@ pub fn make_tuple_type_structure(tb: &mut TokenBuilder, nbr_elements: usize) {
     let PhantomData = ts!(cm.PhantomData "<(" type_params ",)>");
 
     extend_ts!(tb,
-        "#[doc(hidden)]
-        pub struct" cm.TupleN_ident "<" type_params_static_bound "> {
+        "#[doc = " Literal::string(
+            &format!("A marker type implementing [`RefTypes`](crate::mutators::tuples::RefTypes) indicating that a type has the [structure](crate::mutators::tuples::TupleStructure) of a {}-tuple.", nbr_elements)
+        ) "]"
+        "pub struct" cm.TupleN_ident "<" type_params_static_bound "> {
             _phantom: " PhantomData ",
         }
         impl<" type_params_static_bound "> " cm.RefTypes " for " cm.TupleN_ident "<" type_params "> {
@@ -231,27 +234,16 @@ pub(crate) fn impl_default_mutator_for_struct(tb: &mut TokenBuilder, struc: &Str
 #[allow(non_snake_case)]
 fn declare_tuple_mutator(tb: &mut TokenBuilder, nbr_elements: usize) {
     let cm = Common::new(nbr_elements);
-    let Mi = cm.Mi.as_ref();
 
     let mutator_type_params = join_ts!(0..nbr_elements, i, ident!("M" i), separator: ",");
     let type_params = ts!(mutator_type_params);
-
-    let mutator_type_params_replacing_one_by_m = |replacing: usize| -> TokenStream {
-        join_ts!(0..nbr_elements, i, 
-            if i == replacing {
-                ident!("M")
-            } else {
-                Mi(i)
-            }
-        , separator: ",")
-    };
-
     extend_ts!(tb,
+        format!("/// A `TupleMutator` for types that have a {n}-tuple structure", n=nbr_elements)
         "#[derive(" cm.Default ")]"
         "pub struct" cm.TupleNMutator_ident "<" type_params ">"
         "{"
             join_ts!(0..nbr_elements, i,
-                "pub" ident!("mutator_" i) ":" ident!("M" i) ","
+                ident!("mutator_" i) ":" ident!("M" i) ","
             )
             "rng :" cm.fastrand_Rng
         "}
@@ -266,19 +258,6 @@ fn declare_tuple_mutator(tb: &mut TokenBuilder, nbr_elements: usize) {
                     "rng: <_>::default() ,"
                 "}
             }"
-            join_ts!(0..nbr_elements, i,
-                "#[no_coverage]
-                pub fn" ident!("replacing_mutator_" i) " < M > ( self , mutator : M )
-                    ->" cm.TupleNMutator_ident "<" mutator_type_params_replacing_one_by_m(i) " >" "
-                {
-                    " cm.TupleNMutator_ident " {"
-                        join_ts!(0..nbr_elements, j,
-                            ident!("mutator_" j) ":" if i == j { ts!("mutator") } else { ts!("self ." ident!("mutator_" j)) } ","
-                        )
-                        "rng : self.rng ,
-                    }
-                }"
-            )
         "}"
     )
 }
@@ -291,7 +270,9 @@ fn declare_tuple_mutator_helper_types(tb: &mut TokenBuilder, nbr_elements: usize
     let tuple_type_params = join_ts!(0..nbr_elements, i, ident!("T" i), separator: ",");
 
     extend_ts!(tb,
-        "#[derive(" cm.Clone ", " cm.Debug ", " cm.PartialEq ")]
+        "
+        #[doc(hidden)]
+        #[derive(" cm.Clone ")]
         pub struct Cache <" tuple_type_params "> {"
             join_ts!(0..nbr_elements, i,
                 ti(i) ":" ident!("T" i) ","
@@ -299,27 +280,31 @@ fn declare_tuple_mutator_helper_types(tb: &mut TokenBuilder, nbr_elements: usize
             "cplx : f64,
             vose_alias : " cm.VoseAlias "
         }
-        #[derive(" cm.Clone ", " cm.Debug ")]
-        pub enum InnerMutationStep {"
+        #[doc(hidden)]
+        #[derive(" cm.Clone ")]
+        pub enum TupleIndex {"
             join_ts!(0..nbr_elements, i,
                 Ti(i)
             , separator: ",")
         "}
-        #[derive(" cm.Clone ", " cm.Debug ")]
+        #[doc(hidden)]
+        #[derive(" cm.Clone ")]
         pub struct MutationStep < " tuple_type_params " > {"
             join_ts!(0..nbr_elements, i,
                 ti(i) ":" Ti(i) ","
             )
-            "inner : " cm.Vec " < InnerMutationStep > ,
+            "inner : " cm.Vec " < TupleIndex > ,
             vose_alias : Option<" cm.VoseAlias ">
         }
-        #[derive(" cm.Clone ", " cm.Debug ")]
-        pub struct ArbitraryStep < " tuple_type_params " > {"
+        #[doc(hidden)]
+        #[derive(" cm.Clone ")]
+        pub struct RecursingPartIndex < " tuple_type_params " > {"
             join_ts!(0..nbr_elements, i,
-                ti(i) ":" Ti(i)
-            , separator: ",")
-        "}
-
+                ti(i) ":" Ti(i) ","
+            )
+            "inner : " cm.Vec " < TupleIndex >
+        }
+        #[doc(hidden)]
         pub struct UnmutateToken < " tuple_type_params " > {"
             join_ts!(0..nbr_elements, i,
                 "pub" ti(i) ":" cm.Option "<" Ti(i) "> ,"
@@ -357,73 +342,84 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
     let ti_value = cm.ti_value.as_ref();
 
     // let tuple_owned = ts!("(" join_ts!(0..nbr_elements, i, Ti(i), separator: ",") ")");
-    let tuple_ref = ts!("(" join_ts!(0..nbr_elements, i, "&'a" Ti(i) ",") ")");
-    let tuple_mut = ts!("(" join_ts!(0..nbr_elements, i, "&'a mut" Ti(i) ",") ")");
 
     let SelfAsTupleMutator = ts!("<Self as " cm.TupleMutator "<T, " cm.TupleN_ident "<" tuple_type_params "> >>");
 
     let TupleNAsRefTypes = ts!("<" cm.TupleN_ident "<" tuple_type_params "> as " cm.RefTypes ">");
+    let tuple_ref = ts!(
+       TupleNAsRefTypes "::Ref<'a>"
+    );
+    let tuple_mut = ts!(
+       TupleNAsRefTypes "::Mut<'a>"
+    );
 
     extend_ts!(tb,"
     impl <T , " type_params " > " cm.TupleMutator "<T , " cm.TupleN_ident "<" tuple_type_params "> > 
         for " cm.TupleNMutator_ident "< " mutator_type_params " >
     where
-        T: " cm.Clone "," 
+        T: " cm.Clone " + 'static," 
         join_ts!(0..nbr_elements, i,
             Ti(i) ":" cm.Clone " + 'static ,"
             Mi(i) ":" cm.fuzzcheck_traits_Mutator "<" Ti(i) ">,"
         ) "
         T: " cm.TupleStructure "<" cm.TupleN_ident "<" tuple_type_params "> >,
     {
+        #[doc(hidden)]
         type Cache = Cache <"
             join_ts!(0..nbr_elements, i,
                 "<" Mi(i) "as" cm.fuzzcheck_traits_Mutator "<" Ti(i) "> >::Cache "
             , separator: ",")
         ">;
+        #[doc(hidden)]
         type MutationStep = MutationStep <"
             join_ts!(0..nbr_elements, i,
                 "<" Mi(i) "as" cm.fuzzcheck_traits_Mutator "<" Ti(i) "> >::MutationStep "
             , separator: ",")
         ">;
-        type ArbitraryStep = ArbitraryStep <"
+        #[doc(hidden)]
+        type RecursingPartIndex = RecursingPartIndex <"
             join_ts!(0..nbr_elements, i,
-                "<" Mi(i) "as" cm.fuzzcheck_traits_Mutator "<" Ti(i) "> >::ArbitraryStep "
+                "<" Mi(i) "as" cm.fuzzcheck_traits_Mutator "<" Ti(i) "> >::RecursingPartIndex "
             , separator: ",")
         ">;
-        
+
+        #[doc(hidden)]
+        type ArbitraryStep = ();
+
+        #[doc(hidden)]
         type UnmutateToken = UnmutateToken <"
             join_ts!(0..nbr_elements, i,
                 "<" Mi(i) "as" cm.fuzzcheck_traits_Mutator "<" Ti(i) "> >::UnmutateToken "
             , separator: ",")
         ">;
+        #[doc(hidden)]
         #[no_coverage]
         fn default_arbitrary_step(&self) -> Self::ArbitraryStep {
-            Self::ArbitraryStep {"
-                join_ts!(0..nbr_elements, i,
-                    ti(i) ": self." mutator_i(i) ".default_arbitrary_step()"
-                , separator: ",")
-            "}
         }
+        #[doc(hidden)]
         #[no_coverage]
         fn max_complexity(&self) -> f64 {"
             join_ts!(0..nbr_elements, i,
                 "self." mutator_i(i) ".max_complexity()"
             , separator: "+")
         "}
+        #[doc(hidden)]
         #[no_coverage]
         fn min_complexity(&self) -> f64 {"
             join_ts!(0..nbr_elements, i,
                 "self." mutator_i(i) ".min_complexity()"
             , separator: "+")
         "}
+        #[doc(hidden)]
         #[no_coverage]
-        fn complexity<'a>(&'a self, _value: " tuple_ref ", cache: &'a Self::Cache) -> f64 {
+        fn complexity<'a>(&self, _value: " tuple_ref ", cache: &'a Self::Cache) -> f64 {
             cache.cplx
         }
+        #[doc(hidden)]
         #[no_coverage]
-        fn validate_value<'a>(&'a self, value: " tuple_ref ") -> " cm.Option "<(Self::Cache, Self::MutationStep)> {"
+        fn validate_value<'a>(&self, value: " tuple_ref ") -> " cm.Option "<Self::Cache> {"
             join_ts!(0..nbr_elements, i,
-                "let (" ident!("c" i) ", " ident!("s" i) ") = self." mutator_i(i) ".validate_value(value." i ")?;"
+                "let" ident!("c" i) " = self." mutator_i(i) ".validate_value(value." i ")?;"
             )
             join_ts!(0..nbr_elements, i,
                 "let" ident!("cplx_" i) " = self." mutator_i(i) ".complexity(value." i ", &" ident!("c" i) ");"
@@ -439,24 +435,29 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
                     "10. +" ident!("cplx_" i)
                 , separator: ",") "
             ];
-            let sum_prob = probabilities.iter().sum::<f64>();
-            probabilities.iter_mut().for_each(#[no_coverage] |c| *c /= sum_prob);
             let vose_alias = " cm.VoseAlias "::new(probabilities);
 
-            let step = Self::MutationStep {"
-                join_ts!(0..nbr_elements, i, ti(i) ":" ident!("s" i) ",")
-                "inner: vec![" join_ts!(0..nbr_elements, i, "InnerMutationStep::" Ti(i), separator: ",") "] ,
-                vose_alias: Some(vose_alias.clone())
-            };
-
             let cache = Self::Cache {"
                 join_ts!(0..nbr_elements, i, ti(i) ":" ident!("c" i) ",")
                 "cplx: sum_cplx,
                 vose_alias,
             };
 
-            " cm.Some "((cache, step))
+            " cm.Some "(cache)
         }
+        #[doc(hidden)]
+        #[no_coverage]
+        fn default_mutation_step<'a>(&self, value: " tuple_ref ", cache: &'a Self::Cache) -> Self::MutationStep {"
+            join_ts!(0..nbr_elements, i,
+                "let" ident!("s" i) " = self." mutator_i(i) ".default_mutation_step(value." i ", &cache. " ti(i) ");"
+            )"
+             Self::MutationStep {"
+                join_ts!(0..nbr_elements, i, ti(i) ":" ident!("s" i) ",")
+                "inner: vec![" join_ts!(0..nbr_elements, i, "TupleIndex::" Ti(i), separator: ",") "] ,
+                vose_alias: Some(cache.vose_alias.clone())
+            }
+        }
+        #[doc(hidden)]
         #[no_coverage]
         fn ordered_arbitrary(
             &self,
@@ -469,6 +470,7 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
             " // TODO: actually write something that is ordered_arbitrary sense here
             cm.Some "  (self.random_arbitrary(max_cplx))
         }
+        #[doc(hidden)]
         #[no_coverage]
         fn random_arbitrary(&self, max_cplx: f64) -> (T, f64) {"
             join_ts!(0..nbr_elements, i,
@@ -500,9 +502,10 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
                 sum_cplx,
             )
         }
+        #[doc(hidden)]
         #[no_coverage]
         fn ordered_mutate<'a>(
-            &'a self,
+            &self,
             value: " tuple_mut ",
             cache: &'a mut Self::Cache,
             step: &'a mut Self::MutationStep,
@@ -510,7 +513,30 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
         ) -> " cm.Option "<(Self::UnmutateToken, f64)> {
             if max_cplx < <Self as" cm.TupleMutator "<T , " cm.TupleN_ident "<" tuple_type_params "> > >::min_complexity(self) { return " cm.None " }
             if step.inner.is_empty() || step.vose_alias.is_none() {
-                return " cm.None ";
+                let idx1 = self.rng.usize(.." nbr_elements ");
+                let mut idx2 = self.rng.usize(.." nbr_elements " - 1);
+                if idx2 >= idx1 {
+                    idx2 += 1;
+                }
+                assert!(idx1 != idx2);
+                let mut whole_token = Self::UnmutateToken::default();
+                let mut current_cplx = " SelfAsTupleMutator "::complexity(self, " TupleNAsRefTypes "::get_ref_from_mut(&value), cache); 
+                for idx in [idx1, idx2] {
+                    match idx {"
+                    join_ts!(0..nbr_elements, i,
+                        i "=> {
+                            let old_field_cplx = self." mutator_i(i) ".complexity(value." i ", &cache." ti(i) ");
+                            let max_field_cplx = max_cplx - current_cplx + old_field_cplx;
+                            let (token, new_field_cplx) = self." mutator_i(i) "
+                                .random_mutate(value." i ", &mut cache." ti(i) ", max_field_cplx) ;
+                            whole_token. " ti(i) " = " cm.Some "(token);
+                            current_cplx = current_cplx - old_field_cplx + new_field_cplx;
+                        }"
+                    )
+                    "_ => unreachable!()"
+                    "}
+                }
+                return " cm.Some "( (whole_token, current_cplx) );
             }
             let vose_alias = step.vose_alias.as_ref().unwrap();
             let step_idx = vose_alias.sample();
@@ -520,7 +546,7 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
 
             match step.inner[step_idx] {"
             join_ts!(0..nbr_elements, i,
-                "InnerMutationStep::" Ti(i) "=> {
+                "TupleIndex::" Ti(i) "=> {
                     let old_field_cplx = self." mutator_i(i) ".complexity(value." i ", &cache." ti(i) ");
                     let max_field_cplx = max_cplx - current_cplx + old_field_cplx;
                     if let " cm.Some "((token, new_field_cplx)) =
@@ -543,13 +569,13 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
             if sum == 0.0 {
                 step.vose_alias = " cm.None ";
             }  else {
-                prob.iter_mut().for_each(#[no_coverage] |c| *c /= sum );
                 step.vose_alias = " cm.Some "(" cm.VoseAlias "::new(prob));
             }
             " SelfAsTupleMutator "::ordered_mutate(self, value, cache, step, max_cplx)
         }
+        #[doc(hidden)]
         #[no_coverage]
-        fn random_mutate<'a>(&'a self, value: " tuple_mut ", cache: &'a mut Self::Cache, max_cplx: f64, ) -> (Self::UnmutateToken, f64) {
+        fn random_mutate<'a>(&self, value: " tuple_mut ", cache: &'a mut Self::Cache, max_cplx: f64, ) -> (Self::UnmutateToken, f64) {
             let current_cplx = " SelfAsTupleMutator "::complexity(self, " TupleNAsRefTypes "::get_ref_from_mut(&value), cache);
             match cache.vose_alias.sample() {"
                 join_ts!(0..nbr_elements, i,
@@ -568,6 +594,7 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
                 "_ => unreachable!() ,
             }
         }
+        #[doc(hidden)]
         #[no_coverage]
         fn unmutate<'a>(&'a self, value: " tuple_mut ", cache: &'a mut Self::Cache, t: Self::UnmutateToken) {"
             join_ts!(0..nbr_elements, i,
@@ -576,6 +603,42 @@ fn impl_mutator_trait(tb: &mut TokenBuilder, nbr_elements: usize) {
                 }"
             )
         "}
+        #[doc(hidden)]
+        #[no_coverage]
+        fn default_recursing_part_index<'a>(&self, value:" tuple_ref ", cache: &'a Self::Cache) -> Self::RecursingPartIndex {
+            Self::RecursingPartIndex {"
+                join_ts!(0..nbr_elements, i,
+                    ti(i) ": self. " mutator_i(i) ".default_recursing_part_index(&value." i ", &cache." ti(i) ") ,"
+                )
+                "inner: vec![" join_ts!(0..nbr_elements, i, "TupleIndex::" Ti(i), separator: ",") "]
+            }
+        }
+        #[doc(hidden)]
+        #[no_coverage]
+        fn recursing_part<'a, ___V, ___N>(&self, parent: &___N, value:" tuple_ref ", index: &mut Self::RecursingPartIndex) -> " cm.Option " <&'a ___V>
+        where
+            ___V: " cm.Clone " + 'static,
+            ___N: " cm.fuzzcheck_traits_Mutator " <___V>
+        {
+            if index.inner.is_empty() {
+                return " cm.None "
+            }
+            let choice = self.rng.usize(..index.inner.len());
+            let tuple_index = &index.inner[choice];
+            match tuple_index {
+            " join_ts!(0..nbr_elements, i,
+                "TupleIndex::" Ti(i) "=> {
+                    let result = self. " mutator_i(i) ".recursing_part::<___V, ___N>(parent, &value." i ", &mut index." ti(i) ");
+                    if result.is_none() {
+                        index.inner.remove(choice);
+                        " SelfAsTupleMutator "::recursing_part::<___V, ___N>(self, parent, value, index)
+                    } else {
+                        result
+                    }
+                }"
+            )"
+            }
+        }
     }
     "
     )
